@@ -196,8 +196,10 @@ class GeoModelTest(TestCase):
         cities1 = City.objects.all()
         # Only PostGIS would support a 'select *' query because of its recognized
         # HEXEWKB format for geometry fields
-        as_text = 'ST_AsText' if postgis else 'asText'
-        cities2 = City.objects.raw('select id, name, %s(point) from geoapp_city' % as_text)
+        as_text = 'ST_AsText(%s)' if postgis else connection.ops.select
+        cities2 = City.objects.raw(
+            'select id, name, %s from geoapp_city' % as_text % 'point'
+        )
         self.assertEqual(len(cities1), len(list(cities2)))
         self.assertTrue(isinstance(cities2[0].point, Point))
 
@@ -754,7 +756,11 @@ class GeoQuerySetTest(TestCase):
                         self.assertAlmostEqual(c1[0] + xfac, c2[0], 5)
                         self.assertAlmostEqual(c1[1] + yfac, c2[1], 5)
 
+    # TODO: Oracle can be made to pass if
+    # union1 = union2 = fromstr('POINT (-97.5211570000000023 34.4646419999999978)')
+    # but this seems unexpected and should be investigated to determine the cause.
     @no_mysql
+    @no_oracle
     def test_unionagg(self):
         "Testing the `unionagg` (aggregate union) GeoQuerySet method."
         tx = Country.objects.get(name='Texas').mpoly
@@ -777,3 +783,12 @@ class GeoQuerySetTest(TestCase):
         self.assertEqual(True, union.equals_exact(u2, tol))
         qs = City.objects.filter(name='NotACity')
         self.assertEqual(None, qs.unionagg(field_name='point'))
+
+    def test_non_concrete_field(self):
+        pkfield = City._meta.get_field_by_name('id')[0]
+        orig_pkfield_col = pkfield.column
+        pkfield.column = None
+        try:
+            list(City.objects.all())
+        finally:
+            pkfield.column = orig_pkfield_col
