@@ -6,31 +6,36 @@ Usage:
 >>> import datetime
 >>> d = datetime.datetime.now()
 >>> df = DateFormat(d)
->>> print df.format('jS F Y H:i')
+>>> print(df.format('jS F Y H:i'))
 7th October 2003 11:39
 >>>
 """
+from __future__ import unicode_literals
 
 import re
 import time
 import calendar
-from django.utils.dates import MONTHS, MONTHS_3, MONTHS_AP, WEEKDAYS, WEEKDAYS_ABBR
+import datetime
+
+from django.utils.dates import MONTHS, MONTHS_3, MONTHS_ALT, MONTHS_AP, WEEKDAYS, WEEKDAYS_ABBR
 from django.utils.tzinfo import LocalTimezone
 from django.utils.translation import ugettext as _
-from django.utils.encoding import force_unicode
+from django.utils.encoding import force_text
+from django.utils import six
+from django.utils.timezone import is_aware, is_naive
 
-re_formatchars = re.compile(r'(?<!\\)([aAbBdDfFgGhHiIjlLmMnNOPrsStTUwWyYzZ])')
+re_formatchars = re.compile(r'(?<!\\)([aAbBcdDeEfFgGhHiIjlLmMnNoOPrsStTUuwWyYzZ])')
 re_escaped = re.compile(r'\\(.)')
 
 class Formatter(object):
     def format(self, formatstr):
         pieces = []
-        for i, piece in enumerate(re_formatchars.split(force_unicode(formatstr))):
+        for i, piece in enumerate(re_formatchars.split(force_text(formatstr))):
             if i % 2:
-                pieces.append(force_unicode(getattr(self, piece)()))
+                pieces.append(force_text(getattr(self, piece)()))
             elif piece:
                 pieces.append(re_escaped.sub(r'\1', piece))
-        return u''.join(pieces)
+        return ''.join(pieces)
 
 class TimeFormat(Formatter):
     def __init__(self, t):
@@ -61,7 +66,7 @@ class TimeFormat(Formatter):
         """
         if self.data.minute == 0:
             return self.g()
-        return u'%s:%s' % (self.g(), self.i())
+        return '%s:%s' % (self.g(), self.i())
 
     def g(self):
         "Hour, 12-hour format without leading zeros; i.e. '1' to '12'"
@@ -77,15 +82,15 @@ class TimeFormat(Formatter):
 
     def h(self):
         "Hour, 12-hour format; i.e. '01' to '12'"
-        return u'%02d' % self.g()
+        return '%02d' % self.g()
 
     def H(self):
         "Hour, 24-hour format; i.e. '00' to '23'"
-        return u'%02d' % self.G()
+        return '%02d' % self.G()
 
     def i(self):
         "Minutes; i.e. '00' to '59'"
-        return u'%02d' % self.data.minute
+        return '%02d' % self.data.minute
 
     def P(self):
         """
@@ -98,11 +103,16 @@ class TimeFormat(Formatter):
             return _('midnight')
         if self.data.minute == 0 and self.data.hour == 12:
             return _('noon')
-        return u'%s %s' % (self.f(), self.a())
+        return '%s %s' % (self.f(), self.a())
 
     def s(self):
         "Seconds; i.e. '00' to '59'"
-        return u'%02d' % self.data.second
+        return '%02d' % self.data.second
+
+    def u(self):
+        "Microseconds; i.e. '000000' to '999999'"
+        return '%06d' %self.data.microsecond
+
 
 class DateFormat(TimeFormat):
     year_days = [None, 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334]
@@ -110,22 +120,47 @@ class DateFormat(TimeFormat):
     def __init__(self, dt):
         # Accepts either a datetime or date object.
         self.data = dt
-        self.timezone = getattr(dt, 'tzinfo', None)
-        if hasattr(self.data, 'hour') and not self.timezone:
-            self.timezone = LocalTimezone(dt)
+        self.timezone = None
+        if isinstance(dt, datetime.datetime):
+            if is_naive(dt):
+                self.timezone = LocalTimezone(dt)
+            else:
+                self.timezone = dt.tzinfo
 
     def b(self):
         "Month, textual, 3 letters, lowercase; e.g. 'jan'"
         return MONTHS_3[self.data.month]
 
+    def c(self):
+        """
+        ISO 8601 Format
+        Example : '2008-01-02T10:30:00.000123'
+        """
+        return self.data.isoformat()
+
     def d(self):
         "Day of the month, 2 digits with leading zeros; i.e. '01' to '31'"
-        return u'%02d' % self.data.day
+        return '%02d' % self.data.day
 
     def D(self):
         "Day of the week, textual, 3 letters; e.g. 'Fri'"
         return WEEKDAYS_ABBR[self.data.weekday()]
 
+    def e(self):
+        "Timezone name if available"
+        try:
+            if hasattr(self.data, 'tzinfo') and self.data.tzinfo:
+                # Have to use tzinfo.tzname and not datetime.tzname
+                # because datatime.tzname does not expect Unicode
+                return self.data.tzinfo.tzname(self.data) or ""
+        except NotImplementedError:
+            pass
+        return ""
+
+    def E(self):
+        "Alternative month names as required by some locales. Proprietary extension."
+        return MONTHS_ALT[self.data.month]
+
     def F(self):
         "Month, textual, long; e.g. 'January'"
         return MONTHS[self.data.month]
@@ -133,9 +168,9 @@ class DateFormat(TimeFormat):
     def I(self):
         "'1' if Daylight Savings Time, '0' otherwise."
         if self.timezone and self.timezone.dst(self.data):
-            return u'1'
+            return '1'
         else:
-            return u'0'
+            return '0'
 
     def j(self):
         "Day of the month without leading zeros; i.e. '1' to '31'"
@@ -151,7 +186,7 @@ class DateFormat(TimeFormat):
 
     def m(self):
         "Month; i.e. '01' to '12'"
-        return u'%02d' % self.data.month
+        return '%02d' % self.data.month
 
     def M(self):
         "Month, textual, 3 letters; e.g. 'Jan'"
@@ -165,10 +200,16 @@ class DateFormat(TimeFormat):
         "Month abbreviation in Associated Press style. Proprietary extension."
         return MONTHS_AP[self.data.month]
 
+    def o(self):
+        "ISO 8601 year number matching the ISO week number (W)"
+        return self.data.isocalendar()[0]
+
     def O(self):
-        "Difference to Greenwich time in hours; e.g. '+0200'"
+        "Difference to Greenwich time in hours; e.g. '+0200', '-0430'"
         seconds = self.Z()
-        return u"%+03d%02d" % (seconds // 3600, (seconds // 60) % 60)
+        sign = '-' if seconds < 0 else '+'
+        seconds = abs(seconds)
+        return "%s%02d%02d" % (sign, seconds // 3600, (seconds // 60) % 60)
 
     def r(self):
         "RFC 2822 formatted date; e.g. 'Thu, 21 Dec 2000 16:01:07 +0200'"
@@ -177,30 +218,30 @@ class DateFormat(TimeFormat):
     def S(self):
         "English ordinal suffix for the day of the month, 2 characters; i.e. 'st', 'nd', 'rd' or 'th'"
         if self.data.day in (11, 12, 13): # Special case
-            return u'th'
+            return 'th'
         last = self.data.day % 10
         if last == 1:
-            return u'st'
+            return 'st'
         if last == 2:
-            return u'nd'
+            return 'nd'
         if last == 3:
-            return u'rd'
-        return u'th'
+            return 'rd'
+        return 'th'
 
     def t(self):
         "Number of days in the given month; i.e. '28' to '31'"
-        return u'%02d' % calendar.monthrange(self.data.year, self.data.month)[1]
+        return '%02d' % calendar.monthrange(self.data.year, self.data.month)[1]
 
     def T(self):
         "Time zone of this machine; e.g. 'EST' or 'MDT'"
-        name = self.timezone and self.timezone.tzname(self.data) or None
+        name = self.timezone.tzname(self.data) if self.timezone else None
         if name is None:
             name = self.format('O')
-        return unicode(name)
+        return six.text_type(name)
 
     def U(self):
         "Seconds since the Unix epoch (January 1 1970 00:00:00 GMT)"
-        if getattr(self.data, 'tzinfo', None):
+        if isinstance(self.data, datetime.datetime) and is_aware(self.data):
             return int(calendar.timegm(self.data.utctimetuple()))
         else:
             return int(time.mktime(self.data.timetuple()))
@@ -237,7 +278,7 @@ class DateFormat(TimeFormat):
 
     def y(self):
         "Year, 2 digits; e.g. '99'"
-        return unicode(self.data.year)[2:]
+        return six.text_type(self.data.year)[2:]
 
     def Y(self):
         "Year, 4 digits; e.g. '1999'"
@@ -259,8 +300,10 @@ class DateFormat(TimeFormat):
         if not self.timezone:
             return 0
         offset = self.timezone.utcoffset(self.data)
-        # Only days can be negative, so negative offsets have days=-1 and
-        # seconds positive. Positive offsets have days=0
+        # `offset` is a datetime.timedelta. For negative values (to the west of
+        # UTC) only days can be negative (days=-1) and seconds are always
+        # positive. e.g. UTC-1 -> timedelta(days=-1, seconds=82800, microseconds=0)
+        # Positive offsets have days=0
         return offset.days * 86400 + offset.seconds
 
 def format(value, format_string):
